'use strict';

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var fs = _interopDefault(require('fs'));
var path = _interopDefault(require('path'));
var requireUncached = _interopDefault(require('require-uncached'));

// Repeat a string `n` times.
function repeat(str, n) {
  // Use clever algorithm to have O(log(n)) string concatenation operations.
  var res = '';
  while (n > 0) {
    if (n & 1) {
      res += str;
    }
    n >>>= 1;
    str += str;
  }
  return res;
}

// Extend a source object with the properties of another object (shallow copy).
function extend(object, properties) {
  var _iteratorNormalCompletion = true;
  var _didIteratorError = false;
  var _iteratorError = undefined;

  try {
    for (var _iterator = Object.keys(properties)[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
      var key = _step.value;

      var val = properties[key];
      object[key] = val;
    }
  } catch (err) {
    _didIteratorError = true;
    _iteratorError = err;
  } finally {
    try {
      if (!_iteratorNormalCompletion && _iterator.return) {
        _iterator.return();
      }
    } finally {
      if (_didIteratorError) {
        throw _iteratorError;
      }
    }
  }

  return object;
}

var classCallCheck = function (instance, Constructor) {
  if (!(instance instanceof Constructor)) {
    throw new TypeError("Cannot call a class as a function");
  }
};

var createClass = function () {
  function defineProperties(target, props) {
    for (var i = 0; i < props.length; i++) {
      var descriptor = props[i];
      descriptor.enumerable = descriptor.enumerable || false;
      descriptor.configurable = true;
      if ("value" in descriptor) descriptor.writable = true;
      Object.defineProperty(target, descriptor.key, descriptor);
    }
  }

  return function (Constructor, protoProps, staticProps) {
    if (protoProps) defineProperties(Constructor.prototype, protoProps);
    if (staticProps) defineProperties(Constructor, staticProps);
    return Constructor;
  };
}();



























var slicedToArray = function () {
  function sliceIterator(arr, i) {
    var _arr = [];
    var _n = true;
    var _d = false;
    var _e = undefined;

    try {
      for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) {
        _arr.push(_s.value);

        if (i && _arr.length === i) break;
      }
    } catch (err) {
      _d = true;
      _e = err;
    } finally {
      try {
        if (!_n && _i["return"]) _i["return"]();
      } finally {
        if (_d) throw _e;
      }
    }

    return _arr;
  }

  return function (arr, i) {
    if (Array.isArray(arr)) {
      return arr;
    } else if (Symbol.iterator in Object(arr)) {
      return sliceIterator(arr, i);
    } else {
      throw new TypeError("Invalid attempt to destructure non-iterable instance");
    }
  };
}();













var toConsumableArray = function (arr) {
  if (Array.isArray(arr)) {
    for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) arr2[i] = arr[i];

    return arr2;
  } else {
    return Array.from(arr);
  }
};

// Regex matchers for option flags.
var LONG_FLAG = /^(--\w[\w-]*)/;
var SHORT_FLAG = /^(-\w)$/;
var MULTI_FLAG = /^-(\w{2,})/;
var OPTIONAL = /\[(\w+(\*?))\]/;

// A simple **OptionParser** class to parse option flags from the command-line.
// Use it like so:
//
//     parser  = new OptionParser switches, helpBanner
//     options = parser.parse process.argv
//
// The first non-option is considered to be the start of the file (and file
// option) list, and all subsequent arguments are left unparsed.

var OptionParser = function () {

  // Initialize with a list of valid options, in the form:
  //
  //     [short-flag, long-flag, description]
  //
  // Along with an optional banner for the usage help.
  function OptionParser(rules, banner) {
    classCallCheck(this, OptionParser);

    this.banner = banner;
    this.rules = buildRules(rules);
  }

  // Parse the list of arguments, populating an `options` object with all of the
  // specified options, and return it. Options after the first non-option
  // argument are treated as arguments. `options.arguments` will be an array
  // containing the remaining arguments. This is a simpler API than many option
  // parsers that allow you to attach callback actions for every flag. Instead,
  // you're responsible for interpreting the options object.


  createClass(OptionParser, [{
    key: 'parse',
    value: function parse(args) {
      var options = { arguments: [] };
      var skippingArgument = false;
      var originalArgs = args;
      args = normalizeArguments(args);
      for (var i = 0; i < args.length; i++) {
        var arg = args[i];
        if (skippingArgument) {
          skippingArgument = false;
          continue;
        }
        if (arg === '--') {
          var pos = originalArgs.indexOf('--');
          options.arguments = options.arguments.concat(originalArgs.slice(pos + 1));
          break;
        }
        var isOption = !!(arg.match(LONG_FLAG) || arg.match(SHORT_FLAG));
        // the CS option parser is a little odd; options after the first
        // non-option argument are treated as non-option arguments themselves
        var seenNonOptionArg = options.arguments.length > 0;
        if (!seenNonOptionArg) {
          var matchedRule = false;
          var _iteratorNormalCompletion = true;
          var _didIteratorError = false;
          var _iteratorError = undefined;

          try {
            for (var _iterator = Array.from(this.rules)[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
              var rule = _step.value;

              if (rule.shortFlag === arg || rule.longFlag === arg) {
                var value = true;
                if (rule.hasArgument) {
                  skippingArgument = true;
                  value = args[i + 1];
                }
                options[rule.name] = rule.isList ? (options[rule.name] || []).concat(value) : value;
                matchedRule = true;
                break;
              }
            }
          } catch (err) {
            _didIteratorError = true;
            _iteratorError = err;
          } finally {
            try {
              if (!_iteratorNormalCompletion && _iterator.return) {
                _iterator.return();
              }
            } finally {
              if (_didIteratorError) {
                throw _iteratorError;
              }
            }
          }

          if (isOption && !matchedRule) {
            throw new Error('unrecognized option: ' + arg);
          }
        }
        if (seenNonOptionArg || !isOption) {
          options.arguments.push(arg);
        }
      }
      return options;
    }

    // Return the help text for this **OptionParser**, listing and describing all
    // of the valid options, for `--help` and such.

  }, {
    key: 'help',
    value: function help() {
      var lines = [];
      if (this.banner) {
        lines.unshift(this.banner + '\n');
      }
      var _iteratorNormalCompletion2 = true;
      var _didIteratorError2 = false;
      var _iteratorError2 = undefined;

      try {
        for (var _iterator2 = Array.from(this.rules)[Symbol.iterator](), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
          var rule = _step2.value;

          var spaces = 15 - rule.longFlag.length;
          spaces = spaces > 0 ? repeat(' ', spaces) : '';
          var letPart = rule.shortFlag ? rule.shortFlag + ', ' : '    ';
          lines.push('  ' + letPart + rule.longFlag + spaces + rule.description);
        }
      } catch (err) {
        _didIteratorError2 = true;
        _iteratorError2 = err;
      } finally {
        try {
          if (!_iteratorNormalCompletion2 && _iterator2.return) {
            _iterator2.return();
          }
        } finally {
          if (_didIteratorError2) {
            throw _iteratorError2;
          }
        }
      }

      return '\n' + lines.join('\n') + '\n';
    }
  }]);
  return OptionParser;
}();

function buildRules(rules) {
  return function () {
    var result = [];
    var _iteratorNormalCompletion3 = true;
    var _didIteratorError3 = false;
    var _iteratorError3 = undefined;

    try {
      for (var _iterator3 = Array.from(rules)[Symbol.iterator](), _step3; !(_iteratorNormalCompletion3 = (_step3 = _iterator3.next()).done); _iteratorNormalCompletion3 = true) {
        var tuple = _step3.value;

        if (tuple.length < 3) {
          tuple.unshift(null);
        }
        result.push(buildRule.apply(undefined, toConsumableArray(Array.from(tuple || []))));
      }
    } catch (err) {
      _didIteratorError3 = true;
      _iteratorError3 = err;
    } finally {
      try {
        if (!_iteratorNormalCompletion3 && _iterator3.return) {
          _iterator3.return();
        }
      } finally {
        if (_didIteratorError3) {
          throw _iteratorError3;
        }
      }
    }

    return result;
  }();
}

// Build a rule from a `-o` short flag, a `--output [DIR]` long flag, and the
// description of what the option does.
function buildRule(shortFlag, longFlag, description, options) {
  if (options == null) {
    options = {};
  }
  var match = longFlag.match(OPTIONAL);
  longFlag = longFlag.match(LONG_FLAG)[1];
  return {
    name: longFlag.substr(2),
    shortFlag: shortFlag,
    longFlag: longFlag,
    description: description,
    hasArgument: !!(match && match[1]),
    isList: !!(match && match[2])
  };
}

// Normalize arguments by expanding merged flags into multiple flags. This allows
// you to have `-wl` be the same as `--watch --lint`.
function normalizeArguments(args) {
  args = args.slice();
  var result = [];
  var _iteratorNormalCompletion4 = true;
  var _didIteratorError4 = false;
  var _iteratorError4 = undefined;

  try {
    for (var _iterator4 = Array.from(args)[Symbol.iterator](), _step4; !(_iteratorNormalCompletion4 = (_step4 = _iterator4.next()).done); _iteratorNormalCompletion4 = true) {
      var arg = _step4.value;

      var match = void 0;
      if (match = arg.match(MULTI_FLAG)) {
        var _iteratorNormalCompletion5 = true;
        var _didIteratorError5 = false;
        var _iteratorError5 = undefined;

        try {
          for (var _iterator5 = Array.from(match[1].split(''))[Symbol.iterator](), _step5; !(_iteratorNormalCompletion5 = (_step5 = _iterator5.next()).done); _iteratorNormalCompletion5 = true) {
            var l = _step5.value;
            result.push('-' + l);
          }
        } catch (err) {
          _didIteratorError5 = true;
          _iteratorError5 = err;
        } finally {
          try {
            if (!_iteratorNormalCompletion5 && _iterator5.return) {
              _iterator5.return();
            }
          } finally {
            if (_didIteratorError5) {
              throw _iteratorError5;
            }
          }
        }
      } else {
        result.push(arg);
      }
    }
  } catch (err) {
    _didIteratorError4 = true;
    _iteratorError4 = err;
  } finally {
    try {
      if (!_iteratorNormalCompletion4 && _iterator4.return) {
        _iterator4.return();
      }
    } finally {
      if (_didIteratorError4) {
        throw _iteratorError4;
      }
    }
  }

  return result;
}

// Fix any style issues and re-enable lint.
// `cake` is a simplified version of [Make](http://www.gnu.org/software/make/)
// ([Rake](http://rake.rubyforge.org/), [Jake](https://github.com/280north/jake))
// for CoffeeScript. You define tasks with names and descriptions in a Cakefile,
// and can call them from the command line, or invoke them from other tasks.
//
// Running `cake` with no arguments will print out a list of all the tasks in the
// current directory's Cakefile.

// External dependencies.
// Keep track of the list of defined tasks, the accepted options, and so on.
var tasks = {};
var options = {};
var switches = [];
var oparse = null;

// Mixin the top-level Cake functions for Cakefiles to use directly.
extend(global, {

  // Define a Cake task with a short name, an optional sentence description,
  // and the function to run as the action itself.
  task: function task(name, description, action) {
    if (!action) {
      var _Array$from = Array.from([description, action]);

      var _Array$from2 = slicedToArray(_Array$from, 2);

      action = _Array$from2[0];
      description = _Array$from2[1];
    }
    tasks[name] = { name: name, description: description, action: action };
  },


  // Define an option that the Cakefile accepts. The parsed options hash,
  // containing all of the command-line options passed, will be made available
  // as the first argument to the action.
  option: function option(letter, flag, description) {
    switches.push([letter, flag, description]);
  },


  // Invoke another task in the current Cakefile.
  invoke: function invoke(name) {
    if (!tasks[name]) {
      missingTask(name);
    }
    tasks[name].action(options);
  }
});

// Run `cake`. Executes all of the tasks you pass, in order. Note that Node's
// asynchrony may cause tasks to execute in a different order than you'd expect.
// If no tasks are passed, print the help screen. Keep a reference to the
// original directory name, when running Cake tasks from subdirectories.
function run() {
  global.__originalDirname = fs.realpathSync('.');
  process.chdir(cakefileDirectory(__originalDirname));
  var args = process.argv.slice(2);
  require('babel-register'); // eslint-disable-line global-require
  requireUncached(path.resolve('Cakefile'));
  oparse = new OptionParser(switches);
  if (!args.length) {
    return printTasks();
  }
  try {
    options = oparse.parse(args);
  } catch (e) {
    return fatalError('' + e);
  }
  return Array.from(options.arguments).map(function (arg) {
    return invoke(arg);
  });
}

// Display the list of Cake tasks in a format similar to `rake -T`
function printTasks() {
  var relative = path.relative || path.resolve;
  var cakefilePath = path.join(relative(__originalDirname, process.cwd()), 'Cakefile.js');
  console.log(cakefilePath + ' defines the following tasks:\n');
  var _iteratorNormalCompletion = true;
  var _didIteratorError = false;
  var _iteratorError = undefined;

  try {
    for (var _iterator = Object.keys(tasks)[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
      var name = _step.value;

      var task = tasks[name];
      var spaces = 20 - name.length;
      spaces = spaces > 0 ? Array(spaces + 1).join(' ') : '';
      var desc = task.description ? '# ' + task.description : '';
      console.log('cake ' + name + spaces + ' ' + desc);
    }
  } catch (err) {
    _didIteratorError = true;
    _iteratorError = err;
  } finally {
    try {
      if (!_iteratorNormalCompletion && _iterator.return) {
        _iterator.return();
      }
    } finally {
      if (_didIteratorError) {
        throw _iteratorError;
      }
    }
  }

  if (switches.length) {
    console.log(oparse.help());
  }
}

// Print an error and exit when attempting to use an invalid task/option.
function fatalError(message) {
  console.error(message + '\n');
  console.log('To see a list of all tasks/options, run "cake"');
  return process.exit(1);
}

function missingTask(task) {
  fatalError('No such task: ' + task);
}

// When `cake` is invoked, search in the current and all parent directories
// to find the relevant Cakefile.
function cakefileDirectory(dir) {
  if (fs.existsSync(path.join(dir, 'Cakefile.js'))) {
    return dir;
  }
  var parent = path.normalize(path.join(dir, '..'));
  if (parent !== dir) {
    return cakefileDirectory(parent);
  }
  throw new Error('Cakefile not found in ' + process.cwd());
}

module.exports = run;
